﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.SimulationCore {
  [StorableClass]
  public abstract class Activity<TModel> : IActivity<TModel> where TModel : IModel {
    private IEventQueue<TModel> eventQueue;

    [Storable]
    public IEventQueue<TModel> EventQueue {
      get { return eventQueue; }
      protected set { eventQueue = value; }
    }

    public virtual IEnumerable<Type> MonitoredActions {
      get { return new Type[0]; }
    }

    [StorableConstructor]
    protected Activity(bool deserializing) { }
    protected Activity(Activity<TModel> original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      eventQueue = cloner.Clone(original.eventQueue);
    }
    protected Activity(IEventQueue<TModel> queue) {
      eventQueue = queue;
    }

    public abstract void ManageEvents(TModel model, IAction<TModel> lastAction);

    public object Clone() { return Clone(new Cloner()); }
    public abstract IDeepCloneable Clone(Cloner cloner);
  }
}
